@@ -157,5 +157,5 @@ class QueueOutputChannel(CollectingOutputChannel):
     def latest_output(self) -> NoReturn:
         raise NotImplementedError("A queue doesn't allow to peek at messages.")
 
-    async def _persist_message(self, message) -> None:
+    async def _persist_message(self, message: Dict[Text, Any]) -> None:
         await self.messages.put(message)
