@@ -1,5 +1,5 @@
 import logging
-from typing import List, Dict, Text, Optional, Any, Set, TYPE_CHECKING, Tuple
+from typing import Any, List, Dict, Text, Optional, Set, Tuple, TYPE_CHECKING, Union
 
 from tqdm import tqdm
 import numpy as np
@@ -9,7 +9,12 @@ from collections import defaultdict
 from rasa.shared.constants import DOCS_URL_RULES
 from rasa.shared.exceptions import RasaException
 import rasa.shared.utils.io
-from rasa.shared.core.events import LoopInterrupted, UserUttered, ActionExecuted
+from rasa.shared.core.events import (
+    LoopInterrupted,
+    UserUttered,
+    ActionExecuted,
+    Event,
+)
 from rasa.core.featurizers.tracker_featurizers import TrackerFeaturizer
 from rasa.shared.nlu.interpreter import NaturalLanguageInterpreter
 from rasa.core.policies.memoization import MemoizationPolicy
@@ -37,6 +42,8 @@ from rasa.shared.core.constants import (
     LOOP_NAME,
     SLOTS,
     ACTIVE_LOOP,
+    RULE_ONLY_SLOTS,
+    RULE_ONLY_LOOPS,
 )
 from rasa.shared.core.domain import InvalidDomain, State, Domain
 from rasa.shared.nlu.constants import ACTION_NAME, INTENT_NAME_KEY
@@ -58,12 +65,14 @@ DEFAULT_ACTION_MAPPINGS = {
 
 RULES = "rules"
 RULES_FOR_LOOP_UNHAPPY_PATH = "rules_for_loop_unhappy_path"
+RULES_NOT_IN_STORIES = "rules_not_in_stories"
 
 LOOP_WAS_INTERRUPTED = "loop_was_interrupted"
 DO_NOT_PREDICT_LOOP_ACTION = "do_not_predict_loop_action"
 
-DEFAULT_RULES = "predicting default action"
-LOOP_RULES = "handling active loops and forms"
+DEFAULT_RULES = "predicting default action with intent "
+LOOP_RULES = "handling active loops and forms - "
+LOOP_RULES_SEPARATOR = " - "
 
 
 class InvalidRule(RasaException):
@@ -136,7 +145,6 @@ class RulePolicy(MemoizationPolicy):
         self._restrict_rules = restrict_rules
         self._check_for_contradictions = check_for_contradictions
 
-        self._prediction_source = None
         self._rules_sources = None
 
         # max history is set to `None` in order to capture any lengths of rule stories
@@ -404,15 +412,43 @@ class RulePolicy(MemoizationPolicy):
 
         logger.debug("Found no incompletions in rules.")
 
-    def _predict_next_action(
+    @staticmethod
+    def _get_slots_loops_from_states(
+        trackers_as_states: List[List[State]],
+    ) -> Tuple[Set[Text], Set[Text]]:
+        slots = set()
+        loops = set()
+        for states in trackers_as_states:
+            for state in states:
+                slots.update(set(state.get(SLOTS, {}).keys()))
+                active_loop = state.get(ACTIVE_LOOP, {}).get(LOOP_NAME)
+                if active_loop:
+                    loops.add(active_loop)
+        return slots, loops
+
+    def _find_rule_only_slots_loops(
         self,
-        tracker: TrackerWithCachedStates,
-        domain: Domain,
-        interpreter: NaturalLanguageInterpreter,
-    ) -> Optional[Text]:
-        probabilities = self.predict_action_probabilities(
-            tracker, domain, interpreter
-        ).probabilities
+        rule_trackers_as_states: List[List[State]],
+        story_trackers_as_states: List[List[State]],
+    ) -> Tuple[List[Text], List[Text]]:
+        rule_slots, rule_loops = self._get_slots_loops_from_states(
+            rule_trackers_as_states
+        )
+        story_slots, story_loops = self._get_slots_loops_from_states(
+            story_trackers_as_states
+        )
+
+        # set is not json serializable, so convert to list
+        return (
+            list(rule_slots - story_slots - {SHOULD_NOT_BE_SET}),
+            list(rule_loops - story_loops - {SHOULD_NOT_BE_SET}),
+        )
+
+    def _predict_next_action(
+        self, tracker: TrackerWithCachedStates, domain: Domain
+    ) -> Tuple[Optional[Text], Optional[Text]]:
+        prediction, prediction_source = self._predict(tracker, domain)
+        probabilities = prediction.probabilities
         # do not raise an error if RulePolicy didn't predict anything for stories;
         # however for rules RulePolicy should always predict an action
         predicted_action_name = None
@@ -424,18 +460,14 @@ class RulePolicy(MemoizationPolicy):
                 np.argmax(probabilities)
             ]
 
-        return predicted_action_name
+        return predicted_action_name, prediction_source
 
-    def _check_prediction(
-        self,
-        tracker: TrackerWithCachedStates,
-        domain: Domain,
-        interpreter: NaturalLanguageInterpreter,
-        gold_action_name: Text,
-        collect_sources: bool,
-    ) -> Optional[Text]:
-
-        predicted_action_name = self._predict_next_action(tracker, domain, interpreter)
+    def _predicted_action_name(
+        self, tracker: TrackerWithCachedStates, domain: Domain, gold_action_name: Text
+    ) -> Tuple[Optional[Text], Optional[Text]]:
+        predicted_action_name, prediction_source = self._predict_next_action(
+            tracker, domain
+        )
         # if there is an active_loop,
         # RulePolicy will always predict active_loop first,
         # but inside loop unhappy path there might be another action
@@ -445,31 +477,99 @@ class RulePolicy(MemoizationPolicy):
             and predicted_action_name == tracker.active_loop_name
         ):
             rasa.core.test.emulate_loop_rejection(tracker)
-            predicted_action_name = self._predict_next_action(
-                tracker, domain, interpreter
+            predicted_action_name, prediction_source = self._predict_next_action(
+                tracker, domain
             )
 
-        if collect_sources:
-            # we need to remember which action should be predicted by the rule
-            # in order to correctly output the names of the contradicting rules
-            rule_name = tracker.sender_id
-            if self._prediction_source in {DEFAULT_RULES, LOOP_RULES}:
-                # the real gold action contradict the one in the rules in this case
-                gold_action_name = predicted_action_name
-                rule_name = self._prediction_source
+        return predicted_action_name, prediction_source
 
-            self._rules_sources[self._prediction_source].append(
-                (rule_name, gold_action_name)
+    def _collect_sources(
+        self,
+        tracker: TrackerWithCachedStates,
+        predicted_action_name: Optional[Text],
+        gold_action_name: Text,
+        prediction_source: Optional[Text],
+    ) -> None:
+        # we need to remember which action should be predicted by the rule
+        # in order to correctly output the names of the contradicting rules
+        rule_name = tracker.sender_id
+        if prediction_source.startswith(DEFAULT_RULES) or prediction_source.startswith(
+            LOOP_RULES
+        ):
+            # the real gold action contradict the one in the rules in this case
+            gold_action_name = predicted_action_name
+            rule_name = prediction_source
+
+        self._rules_sources[prediction_source].append((rule_name, gold_action_name))
+
+    @staticmethod
+    def _default_sources() -> Set[Text]:
+        return {
+            DEFAULT_RULES + default_intent
+            for default_intent in DEFAULT_ACTION_MAPPINGS.keys()
+        }
+
+    @staticmethod
+    def _handling_loop_sources(domain: Domain) -> Set[Text]:
+        loop_sources = set()
+        for loop_name in domain.form_names:
+            loop_sources.add(LOOP_RULES + loop_name)
+            loop_sources.add(
+                LOOP_RULES + loop_name + LOOP_RULES_SEPARATOR + ACTION_LISTEN_NAME
             )
-            return
+        return loop_sources
 
+    def _should_delete(
+        self,
+        prediction_source: Text,
+        tracker: TrackerWithCachedStates,
+        predicted_action_name: Text,
+    ) -> bool:
+        """Checks whether this contradiction is due to action, intent pair.
+
+        Args:
+            prediction_source: the states that result in the prediction
+            tracker: the tracker that raises the contradiction
+
+        Returns:
+            true if the contradiction is a result of an action, intent pair in the rule.
+        """
+        if (
+            # only apply to contradicting story, not rule
+            tracker.is_rule_tracker
+            # only apply for prediction after unpredictable action
+            or prediction_source.count(PREVIOUS_ACTION) > 1
+            # only apply for prediction of action_listen
+            or predicted_action_name != ACTION_LISTEN_NAME
+        ):
+            return False
+        for source in self.lookup[RULES]:
+            # remove rule only if another action is predicted after action_listen
+            if (
+                source.startswith(prediction_source[:-2])
+                and not prediction_source == source
+            ):
+                return True
+        return False
+
+    def _check_prediction(
+        self,
+        tracker: TrackerWithCachedStates,
+        predicted_action_name: Optional[Text],
+        gold_action_name: Text,
+        prediction_source: Optional[Text],
+    ) -> List[Text]:
         if not predicted_action_name or predicted_action_name == gold_action_name:
-            return
+            return []
+
+        if self._should_delete(prediction_source, tracker, predicted_action_name):
+            self.lookup[RULES].pop(prediction_source)
+            return []
 
         tracker_type = "rule" if tracker.is_rule_tracker else "story"
         contradicting_rules = {
             rule_name
-            for rule_name, action_name in self._rules_sources[self._prediction_source]
+            for rule_name, action_name in self._rules_sources[prediction_source]
             if action_name != gold_action_name
         }
 
@@ -482,19 +582,19 @@ class RulePolicy(MemoizationPolicy):
         if predicted_action_name != self._fallback_action_name:
             error_message += f" which predicted action '{predicted_action_name}'"
 
-        return error_message + "."
+        return [error_message + "."]
 
     def _run_prediction_on_trackers(
         self,
         trackers: List[TrackerWithCachedStates],
         domain: Domain,
-        interpreter: NaturalLanguageInterpreter,
         collect_sources: bool,
-    ) -> List[Text]:
+    ) -> Tuple[List[Text], Set[Text]]:
         if collect_sources:
             self._rules_sources = defaultdict(list)
 
         error_messages = []
+        rules_used_in_stories = set()
         pbar = tqdm(
             trackers,
             desc="Processed trackers",
@@ -524,27 +624,67 @@ class RulePolicy(MemoizationPolicy):
                     continue
 
                 gold_action_name = event.action_name or event.action_text
-                error_message = self._check_prediction(
-                    running_tracker,
-                    domain,
-                    interpreter,
-                    gold_action_name,
-                    collect_sources,
+                predicted_action_name, prediction_source = self._predicted_action_name(
+                    running_tracker, domain, gold_action_name
                 )
-                if error_message:
-                    error_messages.append(error_message)
+                if collect_sources:
+                    self._collect_sources(
+                        running_tracker,
+                        predicted_action_name,
+                        gold_action_name,
+                        prediction_source,
+                    )
+                else:
+                    # to be able to remove only rules turns from the dialogue history
+                    # for ML policies,
+                    # we need to know which rules were used in ML trackers
+                    if (
+                        not tracker.is_rule_tracker
+                        and predicted_action_name == gold_action_name
+                    ):
+                        rules_used_in_stories.add(prediction_source)
+
+                    error_messages += self._check_prediction(
+                        running_tracker,
+                        predicted_action_name,
+                        gold_action_name,
+                        prediction_source,
+                    )
 
                 running_tracker.update(event)
 
-        return error_messages
+        return error_messages, rules_used_in_stories
 
-    def _find_contradicting_rules(
+    def _collect_rule_sources(
+        self, rule_trackers: List[TrackerWithCachedStates], domain: Domain
+    ) -> None:
+        self._run_prediction_on_trackers(rule_trackers, domain, collect_sources=True)
+
+    def _find_contradicting_and_used_in_stories_rules(
+        self, trackers: List[TrackerWithCachedStates], domain: Domain
+    ) -> Tuple[List[Text], Set[Text]]:
+        return self._run_prediction_on_trackers(trackers, domain, collect_sources=False)
+
+    def _analyze_rules(
         self,
         rule_trackers: List[TrackerWithCachedStates],
         all_trackers: List[TrackerWithCachedStates],
         domain: Domain,
-        interpreter: NaturalLanguageInterpreter,
-    ) -> None:
+    ) -> List[Text]:
+        """Analyzes learned rules by running prediction on training trackers.
+
+        This method collects error messages for contradicting rules
+        and creates the lookup for rules that are not present in the stories.
+
+        Args:
+            rule_trackers: The list of the rule trackers.
+            all_trackers: The list of all trackers.
+            domain: The domain.
+            interpreter: Interpreter which can be used by the polices for featurization.
+
+        Returns:
+             Rules that are not present in the stories.
+        """
         logger.debug("Started checking rules and stories for contradictions.")
         # during training we run `predict_action_probabilities` to check for
         # contradicting rules.
@@ -553,13 +693,12 @@ class RulePolicy(MemoizationPolicy):
         logger.setLevel(logging.WARNING)
 
         # we need to run prediction on rule trackers twice, because we need to collect
-        # information which rule snippets contributed to the learned rules
-        self._run_prediction_on_trackers(
-            rule_trackers, domain, interpreter, collect_sources=True
-        )
-        error_messages = self._run_prediction_on_trackers(
-            all_trackers, domain, interpreter, collect_sources=False
-        )
+        # the information about which rule snippets contributed to the learned rules
+        self._collect_rule_sources(rule_trackers, domain)
+        (
+            error_messages,
+            rules_used_in_stories,
+        ) = self._find_contradicting_and_used_in_stories_rules(all_trackers, domain)
 
         logger.setLevel(logger_level)  # reset logger level
         if error_messages:
@@ -571,45 +710,45 @@ class RulePolicy(MemoizationPolicy):
             )
 
         logger.debug("Found no contradicting rules.")
+        all_rules = (
+            set(self._rules_sources.keys())
+            | self._default_sources()
+            | self._handling_loop_sources(domain)
+        )
+        # set is not json serializable, so convert to list
+        return list(all_rules - rules_used_in_stories)
 
-    def train(
+    def _create_lookup_from_trackers(
         self,
-        training_trackers: List[TrackerWithCachedStates],
+        rule_trackers: List[TrackerWithCachedStates],
+        story_trackers: List[TrackerWithCachedStates],
         domain: Domain,
-        interpreter: NaturalLanguageInterpreter,
-        **kwargs: Any,
     ) -> None:
-
-        # only consider original trackers (no augmented ones)
-        training_trackers = [
-            t
-            for t in training_trackers
-            if not hasattr(t, "is_augmented") or not t.is_augmented
-        ]
-        # only use trackers from rule-based training data
-        rule_trackers = [t for t in training_trackers if t.is_rule_tracker]
-        if self._restrict_rules:
-            self._check_rule_restriction(rule_trackers)
-
-        if self._check_for_contradictions:
-            self._check_for_incomplete_rules(rule_trackers, domain)
-
         (
             rule_trackers_as_states,
             rule_trackers_as_actions,
-        ) = self.featurizer.training_states_and_actions(rule_trackers, domain)
+        ) = self.featurizer.training_states_and_actions(
+            rule_trackers, domain, omit_unset_slots=True
+        )
 
         rules_lookup = self._create_lookup_from_states(
             rule_trackers_as_states, rule_trackers_as_actions
         )
         self.lookup[RULES] = self._remove_rule_snippet_predictions(rules_lookup)
 
-        story_trackers = [t for t in training_trackers if not t.is_rule_tracker]
         (
             story_trackers_as_states,
             story_trackers_as_actions,
         ) = self.featurizer.training_states_and_actions(story_trackers, domain)
 
+        if self._check_for_contradictions:
+            (
+                self.lookup[RULE_ONLY_SLOTS],
+                self.lookup[RULE_ONLY_LOOPS],
+            ) = self._find_rule_only_slots_loops(
+                rule_trackers_as_states, story_trackers_as_states
+            )
+
         # use all trackers to find negative rules in unhappy paths
         trackers_as_states = rule_trackers_as_states + story_trackers_as_states
         trackers_as_actions = rule_trackers_as_actions + story_trackers_as_actions
@@ -621,12 +760,42 @@ class RulePolicy(MemoizationPolicy):
             trackers_as_states, trackers_as_actions
         )
 
+    def train(
+        self,
+        training_trackers: List[TrackerWithCachedStates],
+        domain: Domain,
+        interpreter: NaturalLanguageInterpreter,
+        **kwargs: Any,
+    ) -> None:
+        """Trains the policy on given training trackers.
+
+        Args:
+            training_trackers: The list of the trackers.
+            domain: The domain.
+            interpreter: Interpreter which can be used by the polices for featurization.
+        """
+        # only consider original trackers (no augmented ones)
+        training_trackers = [
+            t for t in training_trackers if not getattr(t, "is_augmented", False)
+        ]
+        # trackers from rule-based training data
+        rule_trackers = [t for t in training_trackers if t.is_rule_tracker]
+        if self._restrict_rules:
+            self._check_rule_restriction(rule_trackers)
+        if self._check_for_contradictions:
+            self._check_for_incomplete_rules(rule_trackers, domain)
+
+        # trackers from ML-based training data
+        story_trackers = [t for t in training_trackers if not t.is_rule_tracker]
+
+        self._create_lookup_from_trackers(rule_trackers, story_trackers, domain)
+
         # make this configurable because checking might take a lot of time
         if self._check_for_contradictions:
             # using trackers here might not be the most efficient way, however
             # it allows us to directly test `predict_action_probabilities` method
-            self._find_contradicting_rules(
-                rule_trackers, training_trackers, domain, interpreter
+            self.lookup[RULES_NOT_IN_STORIES] = self._analyze_rules(
+                rule_trackers, training_trackers, domain
             )
 
         logger.debug(f"Memorized '{len(self.lookup[RULES])}' unique rules.")
@@ -635,26 +804,33 @@ class RulePolicy(MemoizationPolicy):
     def _does_rule_match_state(rule_state: State, conversation_state: State) -> bool:
         for state_type, rule_sub_state in rule_state.items():
             conversation_sub_state = conversation_state.get(state_type, {})
-            for key, value in rule_sub_state.items():
-                if isinstance(value, list):
-                    # json dumps and loads tuples as lists,
-                    # so we need to convert them back
-                    value = tuple(value)
+            for key, value_from_rules in rule_sub_state.items():
+                # json dumps and loads tuples as lists
+                if isinstance(value_from_rules, list):
+                    # sort values before comparing
+                    # `sorted` returns a list
+                    value_from_rules = sorted(value_from_rules)
+
+                value_from_conversation = conversation_sub_state.get(key)
+                if isinstance(value_from_conversation, tuple):
+                    # sort values before comparing
+                    # `sorted` returns a list
+                    value_from_conversation = sorted(value_from_conversation)
 
                 if (
                     # value should be set, therefore
                     # check whether it is the same as in the state
-                    value
-                    and value != SHOULD_NOT_BE_SET
-                    and conversation_sub_state.get(key) != value
+                    value_from_rules
+                    and value_from_rules != SHOULD_NOT_BE_SET
+                    and value_from_conversation != value_from_rules
                 ) or (
                     # value shouldn't be set, therefore
                     # it should be None or non existent in the state
-                    value == SHOULD_NOT_BE_SET
-                    and conversation_sub_state.get(key)
+                    value_from_rules == SHOULD_NOT_BE_SET
+                    and value_from_conversation
                     # during training `SHOULD_NOT_BE_SET` is provided. Hence, we also
                     # have to check for the value of the slot state
-                    and conversation_sub_state.get(key) != SHOULD_NOT_BE_SET
+                    and value_from_conversation != SHOULD_NOT_BE_SET
                 ):
                     return False
 
@@ -725,12 +901,12 @@ class RulePolicy(MemoizationPolicy):
     @staticmethod
     def _find_action_from_default_actions(
         tracker: DialogueStateTracker,
-    ) -> Optional[Text]:
+    ) -> Tuple[Optional[Text], Optional[Text]]:
         if (
             not tracker.latest_action_name == ACTION_LISTEN_NAME
             or not tracker.latest_message
         ):
-            return None
+            return None, None
 
         default_action_name = DEFAULT_ACTION_MAPPINGS.get(
             tracker.latest_message.intent.get(INTENT_NAME_KEY)
@@ -738,13 +914,19 @@ class RulePolicy(MemoizationPolicy):
 
         if default_action_name:
             logger.debug(f"Predicted default action '{default_action_name}'.")
+            return (
+                default_action_name,
+                # create prediction source that corresponds to one of
+                # default prediction sources in `_default_sources()`
+                DEFAULT_RULES + tracker.latest_message.intent.get(INTENT_NAME_KEY),
+            )
 
-        return default_action_name
+        return None, None
 
     @staticmethod
     def _find_action_from_loop_happy_path(
         tracker: DialogueStateTracker,
-    ) -> Optional[Text]:
+    ) -> Tuple[Optional[Text], Optional[Text]]:
 
         active_loop_name = tracker.active_loop_name
         active_loop_rejected = tracker.active_loop.get(LOOP_REJECTED)
@@ -761,14 +943,22 @@ class RulePolicy(MemoizationPolicy):
 
         if should_predict_loop:
             logger.debug(f"Predicted loop '{active_loop_name}'.")
-            return active_loop_name
+            return active_loop_name, LOOP_RULES + active_loop_name
 
         # predict `action_listen` if loop action was run successfully
         if should_predict_listen:
             logger.debug(
                 f"Predicted '{ACTION_LISTEN_NAME}' after loop '{active_loop_name}'."
             )
-            return ACTION_LISTEN_NAME
+            return (
+                ACTION_LISTEN_NAME,
+                (
+                    f"{LOOP_RULES}{active_loop_name}"
+                    f"{LOOP_RULES_SEPARATOR}{ACTION_LISTEN_NAME}"
+                ),
+            )
+
+        return None, None
 
     def _find_action_from_rules(
         self,
@@ -798,10 +988,7 @@ class RulePolicy(MemoizationPolicy):
             # the text or the intent
             return None, None, False
 
-        tracker_as_states = self.featurizer.prediction_states(
-            [tracker], domain, use_text_for_last_user_input
-        )
-        states = tracker_as_states[0]
+        states = self._prediction_states(tracker, domain, use_text_for_last_user_input)
 
         current_states = self.format_tracker_states(states)
         logger.debug(f"Current tracker state:{current_states}")
@@ -850,7 +1037,11 @@ class RulePolicy(MemoizationPolicy):
                         f"Predicted loop '{active_loop_name}' by overwriting "
                         f"'{ACTION_LISTEN_NAME}' predicted by general rule."
                     )
-                    return active_loop_name, LOOP_RULES, returning_from_unhappy_path
+                    return (
+                        active_loop_name,
+                        best_rule_key,
+                        returning_from_unhappy_path,
+                    )
 
                 # do not predict anything
                 predicted_action_name = None
@@ -883,11 +1074,17 @@ class RulePolicy(MemoizationPolicy):
         domain: Domain,
         interpreter: NaturalLanguageInterpreter,
         **kwargs: Any,
-    ) -> PolicyPrediction:
+    ) -> "PolicyPrediction":
         """Predicts the next action (see parent class for more information)."""
+        prediction, _ = self._predict(tracker, domain)
+        return prediction
+
+    def _predict(
+        self, tracker: DialogueStateTracker, domain: Domain
+    ) -> Tuple[PolicyPrediction, Text]:
         (
             rules_action_name_from_text,
-            self._prediction_source,
+            prediction_source_from_text,
             returning_from_unhappy_path_from_text,
         ) = self._find_action_from_rules(
             tracker, domain, use_text_for_last_user_input=True
@@ -896,42 +1093,62 @@ class RulePolicy(MemoizationPolicy):
         # Rasa Open Source default actions overrule anything. If users want to achieve
         # the same, they need to write a rule or make sure that their loop rejects
         # accordingly.
-        default_action_name = self._find_action_from_default_actions(tracker)
+        (
+            default_action_name,
+            default_prediction_source,
+        ) = self._find_action_from_default_actions(tracker)
 
         # text has priority over intents including default,
         # however loop happy path has priority over rules prediction
         if default_action_name and not rules_action_name_from_text:
-            self._prediction_source = DEFAULT_RULES
-            return self._prediction(
-                self._prediction_result(default_action_name, tracker, domain)
+            return (
+                self._rule_prediction(
+                    self._prediction_result(default_action_name, tracker, domain),
+                    default_prediction_source,
+                ),
+                default_prediction_source,
             )
 
         # A loop has priority over any other rule except defaults.
         # The rules or any other prediction will be applied only if a loop was rejected.
         # If we are in a loop, and the loop didn't run previously or rejected, we can
         # simply force predict the loop.
-        loop_happy_path_action_name = self._find_action_from_loop_happy_path(tracker)
+        (
+            loop_happy_path_action_name,
+            loop_happy_path_prediction_source,
+        ) = self._find_action_from_loop_happy_path(tracker)
         if loop_happy_path_action_name:
-            self._prediction_source = LOOP_RULES
             # this prediction doesn't use user input
             # and happy user input anyhow should be ignored during featurization
-            return self._prediction(
-                self._prediction_result(loop_happy_path_action_name, tracker, domain),
-                is_no_user_prediction=True,
+            return (
+                self._rule_prediction(
+                    self._prediction_result(
+                        loop_happy_path_action_name, tracker, domain
+                    ),
+                    loop_happy_path_prediction_source,
+                    is_no_user_prediction=True,
+                ),
+                loop_happy_path_prediction_source,
             )
 
         # predict rules from text first
         if rules_action_name_from_text:
-            return self._prediction_with_unhappy_path(
-                self._prediction_result(rules_action_name_from_text, tracker, domain),
-                returning_from_unhappy_path=returning_from_unhappy_path_from_text,
-                is_end_to_end_prediction=True,
+            return (
+                self._rule_prediction(
+                    self._prediction_result(
+                        rules_action_name_from_text, tracker, domain
+                    ),
+                    prediction_source_from_text,
+                    returning_from_unhappy_path=returning_from_unhappy_path_from_text,
+                    is_end_to_end_prediction=True,
+                ),
+                prediction_source_from_text,
             )
 
         (
             rules_action_name_from_intent,
             # we want to remember the source even if rules didn't predict any action
-            self._prediction_source,
+            prediction_source_from_intent,
             returning_from_unhappy_path_from_intent,
         ) = self._find_action_from_rules(
             tracker, domain, use_text_for_last_user_input=False
@@ -943,27 +1160,41 @@ class RulePolicy(MemoizationPolicy):
         else:
             probabilities = self._default_predictions(domain)
 
-        return self._prediction_with_unhappy_path(
-            probabilities,
-            returning_from_unhappy_path=(
-                # returning_from_unhappy_path is a negative condition,
-                # so `or` should be applied
-                returning_from_unhappy_path_from_text
-                or returning_from_unhappy_path_from_intent
+        return (
+            self._rule_prediction(
+                probabilities,
+                prediction_source_from_intent,
+                returning_from_unhappy_path=(
+                    # returning_from_unhappy_path is a negative condition,
+                    # so `or` should be applied
+                    returning_from_unhappy_path_from_text
+                    or returning_from_unhappy_path_from_intent
+                ),
+                is_end_to_end_prediction=False,
             ),
-            is_end_to_end_prediction=False,
+            prediction_source_from_intent,
         )
 
-    def _prediction_with_unhappy_path(
+    def _rule_prediction(
         self,
         probabilities: List[float],
-        returning_from_unhappy_path: bool,
-        is_end_to_end_prediction: bool,
-    ) -> "PolicyPrediction":
-        return self._prediction(
+        prediction_source: Text,
+        returning_from_unhappy_path: bool = False,
+        is_end_to_end_prediction: bool = False,
+        is_no_user_prediction: bool = False,
+    ) -> PolicyPrediction:
+        return PolicyPrediction(
             probabilities,
+            self.__class__.__name__,
+            self.priority,
             events=[LoopInterrupted(True)] if returning_from_unhappy_path else [],
             is_end_to_end_prediction=is_end_to_end_prediction,
+            is_no_user_prediction=is_no_user_prediction,
+            hide_rule_turn=(
+                True
+                if prediction_source in self.lookup.get(RULES_NOT_IN_STORIES, [])
+                else False
+            ),
         )
 
     def _default_predictions(self, domain: Domain) -> List[float]:
@@ -987,3 +1218,13 @@ class RulePolicy(MemoizationPolicy):
     @classmethod
     def _metadata_filename(cls) -> Text:
         return "rule_policy.json"
+
+    def get_rule_only_data(self) -> Dict[Text, Any]:
+        """Gets the slots and loops that are used only in rule data.
+
+        Returns:
+            Slots and loops that are used only in rule data.
+        """
+        return {
+            key: self.lookup.get(key, []) for key in [RULE_ONLY_SLOTS, RULE_ONLY_LOOPS]
+        }
