@@ -6,11 +6,24 @@ import textwrap
 import uuid
 from functools import partial
 from multiprocessing import Process
-from typing import Any, Callable, Dict, List, Optional, Text, Tuple, Union, Set
-
+from typing import (
+    Any,
+    Callable,
+    Deque,
+    Dict,
+    List,
+    Optional,
+    Text,
+    Tuple,
+    Union,
+    Set,
+    Coroutine,
+)
 
 from sanic import Sanic, response
 from sanic.exceptions import NotFound
+from sanic.request import Request
+from sanic.response import HTTPResponse
 from terminaltables import AsciiTable, SingleTable
 import numpy as np
 from aiohttp import ClientError
@@ -34,7 +47,8 @@ from rasa.shared.core.constants import (
     REQUESTED_SLOT,
     LOOP_INTERRUPTED,
 )
-from rasa.core import run, train, utils
+from rasa.core import run, utils
+import rasa.core.train
 from rasa.core.constants import DEFAULT_SERVER_FORMAT, DEFAULT_SERVER_PORT
 from rasa.shared.core.domain import Domain
 import rasa.shared.core.events
@@ -393,7 +407,7 @@ async def _request_fork_point_from_list(
 
 
 async def _request_fork_from_user(
-    conversation_id, endpoint
+    conversation_id: Text, endpoint: EndpointConfig
 ) -> Optional[List[Dict[Text, Any]]]:
     """Take in a conversation and ask at which point to fork the conversation.
 
@@ -420,7 +434,10 @@ async def _request_fork_from_user(
 
 
 async def _request_intent_from_user(
-    latest_message, intents, conversation_id, endpoint
+    latest_message: Dict[Text, Any],
+    intents: List[Text],
+    conversation_id: Text,
+    endpoint: EndpointConfig,
 ) -> Dict[Text, Any]:
     """Take in latest message and ask which intent it should have been.
 
@@ -508,10 +525,10 @@ def _chat_history_table(events: List[Dict[Text, Any]]) -> Text:
     def user_width(_table: AsciiTable) -> int:
         return _table.column_max_width(3)
 
-    def add_bot_cell(data, cell):
+    def add_bot_cell(data: List[List[Union[Text, Color]]], cell: Text) -> None:
         data.append([len(data), Color(cell), "", ""])
 
-    def add_user_cell(data, cell):
+    def add_user_cell(data: List[List[Union[Text, Color]]], cell: Text) -> None:
         data.append([len(data), "", "", Color(cell)])
 
     # prints the historical interactions between the bot and the user,
@@ -595,7 +612,7 @@ def _retry_on_error(
                 raise e
 
 
-async def _write_data_to_file(conversation_id: Text, endpoint: EndpointConfig):
+async def _write_data_to_file(conversation_id: Text, endpoint: EndpointConfig) -> None:
     """Write stories and nlu data to file."""
 
     story_path, nlu_path, domain_path = _request_export_info()
@@ -1061,7 +1078,10 @@ def _form_is_restored(action_name: Text, tracker: Dict[Text, Any]) -> bool:
 
 
 async def _confirm_form_validation(
-    action_name, tracker, endpoint, conversation_id
+    action_name: Text,
+    tracker: Dict[Text, Any],
+    endpoint: EndpointConfig,
+    conversation_id: Text,
 ) -> None:
     """Ask a user whether an input for a form should be validated.
 
@@ -1523,7 +1543,7 @@ async def record_messages(
 
 async def _get_tracker_events_to_plot(
     domain: Dict[Text, Any], file_importer: TrainingDataImporter, conversation_id: Text
-) -> List[Union[Text, List[Event]]]:
+) -> List[Union[Text, Deque[Event]]]:
     training_trackers = await _get_training_trackers(file_importer, domain)
     number_of_trackers = len(training_trackers)
     if number_of_trackers > MAX_NUMBER_OF_TRAINING_STORIES_FOR_VISUALIZATION:
@@ -1536,10 +1556,10 @@ async def _get_tracker_events_to_plot(
         )
         training_trackers = []
 
-    training_data_events = [t.events for t in training_trackers]
-    events_including_current_user_id = training_data_events + [conversation_id]
-
-    return events_including_current_user_id
+    training_data_events: List[Union[Text, Deque[Event]]] = [
+        t.events for t in training_trackers
+    ]
+    return training_data_events + [conversation_id]
 
 
 async def _get_training_trackers(
@@ -1596,17 +1616,17 @@ def start_visualization(image_path: Text, port: int) -> None:
 
     # noinspection PyUnusedLocal
     @app.exception(NotFound)
-    async def ignore_404s(request, exception):
+    async def ignore_404s(request: Request, exception: Exception) -> HTTPResponse:
         return response.text("Not found", status=404)
 
     # noinspection PyUnusedLocal
     @app.route(VISUALIZATION_TEMPLATE_PATH, methods=["GET"])
-    def visualisation_html(request):
+    def visualisation_html(request: Request) -> HTTPResponse:
         return response.file(visualization.visualization_html_path())
 
     # noinspection PyUnusedLocal
     @app.route("/visualization.dot", methods=["GET"])
-    def visualisation_png(request):
+    def visualisation_png(request: Request,) -> HTTPResponse:
         try:
             headers = {"Cache-Control": "no-cache"}
             return response.file(os.path.abspath(image_path), headers=headers)
@@ -1620,7 +1640,11 @@ def start_visualization(image_path: Text, port: int) -> None:
 
 # noinspection PyUnusedLocal
 async def train_agent_on_start(
-    args, endpoints, additional_arguments, app, loop
+    args: Dict[Text, Any],
+    endpoints: AvailableEndpoints,
+    additional_arguments: Optional[Dict],
+    app: Sanic,
+    loop: asyncio.AbstractEventLoop,
 ) -> None:
     _interpreter = rasa.core.interpreter.create_interpreter(
         endpoints.nlu or args.get("nlu")
@@ -1628,7 +1652,7 @@ async def train_agent_on_start(
 
     model_directory = args.get("out", tempfile.mkdtemp(suffix="_core_model"))
 
-    _agent = await train(
+    _agent = await rasa.core.train.train(
         args.get("domain"),
         args.get("stories"),
         model_directory,
@@ -1642,10 +1666,9 @@ async def train_agent_on_start(
 
 
 async def wait_til_server_is_running(
-    endpoint, max_retries=30, sleep_between_retries=1
+    endpoint: EndpointConfig, max_retries: int = 30, sleep_between_retries: float = 1.0
 ) -> bool:
     """Try to reach the server, retry a couple of times and sleep in between."""
-
     while max_retries:
         try:
             r = await retrieve_status(endpoint)
