@@ -208,7 +208,9 @@ def validate_pipeline(pipeline: List["Component"]) -> None:
     validate_required_components(pipeline)
 
 
-def any_components_in_pipeline(components: Iterable[Text], pipeline: List["Component"]):
+def any_components_in_pipeline(
+    components: Iterable[Text], pipeline: List["Component"]
+) -> bool:
     """Check if any of the provided components are listed in the pipeline.
 
     Args:
@@ -361,9 +363,8 @@ class ComponentMetaclass(type):
     """Metaclass with `name` class property."""
 
     @property
-    def name(cls):
+    def name(cls) -> Text:
         """The name property is a function of the class - its __name__."""
-
         return cls.__name__
 
 
@@ -483,7 +484,7 @@ class Component(metaclass=ComponentMetaclass):
     def load(
         cls,
         meta: Dict[Text, Any],
-        model_dir: Optional[Text] = None,
+        model_dir: Text,
         model_metadata: Optional["Metadata"] = None,
         cached_component: Optional["Component"] = None,
         **kwargs: Any,
@@ -636,7 +637,7 @@ class Component(metaclass=ComponentMetaclass):
             del d["partial_processing_pipeline"]
         return d
 
-    def __eq__(self, other) -> bool:
+    def __eq__(self, other: Any) -> bool:
         return self.__dict__ == other.__dict__
 
     def prepare_partial_processing(
@@ -855,7 +856,7 @@ class ComponentBuilder:
 
         try:
             component, cache_key = self.__get_cached_component(
-                component_config, Metadata(cfg.as_dict(), None)
+                component_config, Metadata(cfg.as_dict())
             )
             if component is None:
                 component = registry.create_component_by_config(component_config, cfg)
