@@ -4,6 +4,8 @@ DOCS_URL_TRAINING_DATA = DOCS_BASE_URL + "/training-data-format"
 DOCS_URL_TRAINING_DATA_NLU = DOCS_URL_TRAINING_DATA + "#nlu-training-data"
 DOCS_URL_DOMAINS = DOCS_BASE_URL + "/domain"
 DOCS_URL_SLOTS = DOCS_URL_DOMAINS + "#slots"
+DOCS_URL_INTENTS = DOCS_URL_DOMAINS + "#intents"
+DOCS_URL_ENTITIES = DOCS_URL_DOMAINS + "#entities"
 DOCS_URL_RESPONSES = DOCS_BASE_URL + "/responses"
 DOCS_URL_STORIES = DOCS_BASE_URL + "/stories"
 DOCS_URL_RULES = DOCS_BASE_URL + "/rules"
@@ -79,5 +81,6 @@ DEFAULT_RESULTS_PATH = "results"
 DEFAULT_NLU_RESULTS_PATH = "nlu_comparison_results"
 DEFAULT_CORE_SUBDIRECTORY_NAME = "core"
 DEFAULT_NLU_SUBDIRECTORY_NAME = "nlu"
+DEFAULT_CONVERSATION_TEST_PATH = "tests"
 
 DIAGNOSTIC_DATA = "diagnostic_data"
