@@ -9,7 +9,18 @@ import time
 import uuid
 from dateutil import parser
 from datetime import datetime
-from typing import List, Dict, Text, Any, Type, Optional, TYPE_CHECKING, Iterable
+from typing import (
+    List,
+    Dict,
+    Text,
+    Any,
+    Type,
+    Optional,
+    TYPE_CHECKING,
+    Iterable,
+    cast,
+    Tuple,
+)
 
 import rasa.shared.utils.common
 from typing import Union
@@ -432,7 +443,7 @@ class UserUttered(Event):
         input_channel: Optional[Text] = None,
         message_id: Optional[Text] = None,
         metadata: Optional[Dict] = None,
-    ):
+    ) -> "UserUttered":
         return UserUttered(
             text,
             parse_data.get(INTENT),
@@ -685,7 +696,7 @@ class DefinePrevUserUtteredFeaturization(SkipEventInMDStoryMixin):
             self.use_text_for_featurization
         )
 
-    def __eq__(self, other) -> bool:
+    def __eq__(self, other: Any) -> bool:
         """Compares object with other object."""
         if not isinstance(other, DefinePrevUserUtteredFeaturization):
             return NotImplemented
@@ -726,7 +737,10 @@ class EntitiesAdded(SkipEventInMDStoryMixin):
 
     def __eq__(self, other: Any) -> bool:
         """Compares this event with another event."""
-        return isinstance(other, EntitiesAdded)
+        if not isinstance(other, EntitiesAdded):
+            return NotImplemented
+
+        return self.entities == other.entities
 
     @classmethod
     def _from_parameters(cls, parameters: Dict[Text, Any]) -> "EntitiesAdded":
@@ -772,7 +786,13 @@ class BotUttered(SkipEventInMDStoryMixin):
 
     type_name = "bot"
 
-    def __init__(self, text=None, data=None, metadata=None, timestamp=None) -> None:
+    def __init__(
+        self,
+        text: Optional[Text] = None,
+        data: Optional[Dict] = None,
+        metadata: Optional[Dict[Text, Any]] = None,
+        timestamp: Optional[float] = None,
+    ) -> None:
         """Creates event for a bot response.
 
         Args:
@@ -785,7 +805,7 @@ class BotUttered(SkipEventInMDStoryMixin):
         self.data = data or {}
         super().__init__(timestamp, metadata)
 
-    def __members(self):
+    def __members(self) -> Tuple[Optional[Text], Text, Text]:
         data_no_nones = {k: v for k, v in self.data.items() if v is not None}
         meta_no_nones = {k: v for k, v in self.metadata.items() if v is not None}
         return (
@@ -798,7 +818,7 @@ class BotUttered(SkipEventInMDStoryMixin):
         """Returns unique hash for event."""
         return hash(self.__members())
 
-    def __eq__(self, other) -> bool:
+    def __eq__(self, other: Any) -> bool:
         """Compares object with other object."""
         if not isinstance(other, BotUttered):
             return NotImplemented
@@ -849,7 +869,7 @@ class BotUttered(SkipEventInMDStoryMixin):
         return d
 
     @classmethod
-    def _from_parameters(cls, parameters) -> "BotUttered":
+    def _from_parameters(cls, parameters: Dict[Text, Any]) -> "BotUttered":
         try:
             return BotUttered(
                 parameters.get("text"),
@@ -900,7 +920,7 @@ class SlotSet(Event):
         """Returns unique hash for event."""
         return hash((self.key, jsonpickle.encode(self.value)))
 
-    def __eq__(self, other) -> bool:
+    def __eq__(self, other: Any) -> bool:
         """Compares object with other object."""
         if not isinstance(other, SlotSet):
             return NotImplemented
@@ -931,7 +951,7 @@ class SlotSet(Event):
         return d
 
     @classmethod
-    def _from_parameters(cls, parameters) -> "SlotSet":
+    def _from_parameters(cls, parameters: Dict[Text, Any]) -> "SlotSet":
         try:
             return SlotSet(
                 parameters.get("name"),
@@ -1013,7 +1033,7 @@ class AllSlotsReset(AlwaysEqualEventMixin):
         """Returns text representation of event."""
         return self.type_name
 
-    def apply_to(self, tracker) -> None:
+    def apply_to(self, tracker: "DialogueStateTracker") -> None:
         """Applies event to current conversation state."""
         tracker._reset_slots()
 
@@ -1308,7 +1328,7 @@ class StoryExported(Event):
         if self.path:
             tracker.export_stories_to_file(self.path)
 
-    def __eq__(self, other) -> bool:
+    def __eq__(self, other: Any) -> bool:
         """Compares object with other object."""
         if not isinstance(other, StoryExported):
             return NotImplemented
@@ -1341,7 +1361,7 @@ class FollowupAction(Event):
         """Returns unique hash for event."""
         return hash(self.action_name)
 
-    def __eq__(self, other) -> bool:
+    def __eq__(self, other: Any) -> bool:
         """Compares object with other object."""
         if not isinstance(other, FollowupAction):
             return NotImplemented
@@ -1396,7 +1416,7 @@ class ConversationPaused(AlwaysEqualEventMixin):
         """Returns text representation of event."""
         return str(self)
 
-    def apply_to(self, tracker) -> None:
+    def apply_to(self, tracker: "DialogueStateTracker") -> None:
         """Applies event to current conversation state."""
         tracker._paused = True
 
@@ -1418,7 +1438,7 @@ class ConversationResumed(AlwaysEqualEventMixin):
         """Returns text representation of event."""
         return self.type_name
 
-    def apply_to(self, tracker) -> None:
+    def apply_to(self, tracker: "DialogueStateTracker") -> None:
         """Applies event to current conversation state."""
         tracker._paused = False
 
@@ -1440,6 +1460,7 @@ class ActionExecuted(Event):
         timestamp: Optional[float] = None,
         metadata: Optional[Dict] = None,
         action_text: Optional[Text] = None,
+        hide_rule_turn: bool = False,
     ) -> None:
         """Creates event for a successful event execution.
 
@@ -1452,12 +1473,15 @@ class ActionExecuted(Event):
             metadata: Additional event metadata.
             action_text: In case it's an end-to-end action prediction, the text which
                 was predicted.
+            hide_rule_turn: If `True`, this action should be hidden in the dialogue
+                history created for ML-based policies.
         """
         self.action_name = action_name
         self.policy = policy
         self.confidence = confidence
         self.unpredictable = False
         self.action_text = action_text
+        self.hide_rule_turn = hide_rule_turn
 
         super().__init__(timestamp, metadata)
 
@@ -1499,7 +1523,6 @@ class ActionExecuted(Event):
 
     @classmethod
     def _from_story_string(cls, parameters: Dict[Text, Any]) -> Optional[List[Event]]:
-
         return [
             ActionExecuted(
                 parameters.get("name"),
@@ -1508,25 +1531,20 @@ class ActionExecuted(Event):
                 parameters.get("timestamp"),
                 parameters.get("metadata"),
                 parameters.get("action_text"),
+                parameters.get("hide_rule_turn"),
             )
         ]
 
     def as_dict(self) -> Dict[Text, Any]:
         """Returns serialized event."""
         d = super().as_dict()
-        policy = None  # for backwards compatibility (persisted events)
-        if hasattr(self, "policy"):
-            policy = self.policy
-        confidence = None
-        if hasattr(self, "confidence"):
-            confidence = self.confidence
-
         d.update(
             {
                 "name": self.action_name,
-                "policy": policy,
-                "confidence": confidence,
+                "policy": self.policy,
+                "confidence": self.confidence,
                 "action_text": self.action_text,
+                "hide_rule_turn": self.hide_rule_turn,
             }
         )
         return d
@@ -1543,7 +1561,9 @@ class ActionExecuted(Event):
         if self.action_name:
             return {ACTION_NAME: self.action_name}
         else:
-            return {ACTION_TEXT: self.action_text}
+            # FIXME: we should define the type better here, and require either
+            #        `action_name` or `action_text`
+            return {ACTION_TEXT: cast(Text, self.action_text)}
 
     def apply_to(self, tracker: "DialogueStateTracker") -> None:
         """Applies event to current conversation state."""
@@ -1576,7 +1596,7 @@ class AgentUttered(SkipEventInMDStoryMixin):
         """Returns unique hash for event."""
         return hash((self.text, jsonpickle.encode(self.data)))
 
-    def __eq__(self, other) -> bool:
+    def __eq__(self, other: Any) -> bool:
         """Compares object with other object."""
         if not isinstance(other, AgentUttered):
             return NotImplemented
@@ -1599,7 +1619,7 @@ class AgentUttered(SkipEventInMDStoryMixin):
         return d
 
     @classmethod
-    def _from_parameters(cls, parameters) -> "AgentUttered":
+    def _from_parameters(cls, parameters: Dict[Text, Any]) -> "AgentUttered":
         try:
             return AgentUttered(
                 parameters.get("text"),
@@ -1640,7 +1660,7 @@ class ActiveLoop(Event):
         """Returns unique hash for event."""
         return hash(self.name)
 
-    def __eq__(self, other) -> bool:
+    def __eq__(self, other: Any) -> bool:
         """Compares object with other object."""
         if not isinstance(other, ActiveLoop):
             return NotImplemented
@@ -1728,7 +1748,7 @@ class LoopInterrupted(SkipEventInMDStoryMixin):
         """Returns unique hash for event."""
         return hash(self.is_interrupted)
 
-    def __eq__(self, other) -> bool:
+    def __eq__(self, other: Any) -> bool:
         """Compares object with other object."""
         if not isinstance(other, LoopInterrupted):
             return NotImplemented
@@ -1736,7 +1756,7 @@ class LoopInterrupted(SkipEventInMDStoryMixin):
         return self.is_interrupted == other.is_interrupted
 
     @classmethod
-    def _from_parameters(cls, parameters) -> "LoopInterrupted":
+    def _from_parameters(cls, parameters: Dict[Text, Any]) -> "LoopInterrupted":
         return LoopInterrupted(
             parameters.get(LOOP_INTERRUPTED, False),
             parameters.get("timestamp"),
@@ -1831,7 +1851,7 @@ class ActionExecutionRejected(SkipEventInMDStoryMixin):
         """Returns unique hash for event."""
         return hash(self.action_name)
 
-    def __eq__(self, other) -> bool:
+    def __eq__(self, other: Any) -> bool:
         """Compares object with other object."""
         if not isinstance(other, ActionExecutionRejected):
             return NotImplemented
@@ -1839,7 +1859,7 @@ class ActionExecutionRejected(SkipEventInMDStoryMixin):
         return self.action_name == other.action_name
 
     @classmethod
-    def _from_parameters(cls, parameters) -> "ActionExecutionRejected":
+    def _from_parameters(cls, parameters: Dict[Text, Any]) -> "ActionExecutionRejected":
         return ActionExecutionRejected(
             parameters.get("name"),
             parameters.get("policy"),
